import pandas as pd
import numpy as np
import pickle
import copy

from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
from xgboost import XGBClassifier, XGBRegressor
from sklearn.linear_model import LinearRegression


class NingXiang:

    def __init__(self):
        """Initialise class"""

        self._initialise_objects()

        print("NingXiang Initialised")

    def _initialise_objects(self):

        self._seed = 42
        self.train_x = None
        self.train_y = None
        self.val_x = None
        self.val_y = None
        self.clf_type = None
        self.ningxiang_output = None
        self.object_saving_address = None
        self.feature_importance = None

    def read_in_train_data(self, train_x, train_y, val_x=None, val_y=None):
        """Reads in train data for building NingXiang output object"""

        self.train_x = train_x
        print("Read in Train X data")

        self.train_y = train_y
        print("Read in Train y data")

        if val_x is not None:
            self.val_x = val_x
            print("Read in Val X data")

        if val_y is not None:
            self.val_y = val_y
            print("Read in Val y data")

    def set_model_type(self, type):
        """Reads in underlying model object for tuning, and also read in what type of model it is"""

        assert (
            type == "Classification" or type == "Regression"
        ), 'Please input either "Classification" or "Regression"'

        self.clf_type = type

        print(f"Successfully recorded model type: {self.clf_type}")

    def get_lr_based_feature_combinations(self, min_features=0, gap=1):
        """Get NingXiang scores based on LR feature importance"""

        if self.clf_type == None:
            raise AttributeError(
                "clf_type not found, please run set_model_type() first"
            )

        if self.clf_type == "Classification":
            raise AttributeError(
                ".get_lr_based_combinations() only works for continuous labels"
            )

        if self.train_x is None or self.train_y is None:
            raise AttributeError(
                "train_x and train_y not found, please run read_in_train_data"
            )

        self.ningxiang_output = dict()

        curr_combo = list()
        remaining_features = list(self.train_x.columns)
        step = 0
        for i in range(len(remaining_features)):
            print(f"Up to {i+1}th variable")

            best_score = 0
            best_combo = None

            # try adding each new feature and getting lr
            for feature in remaining_features:
                tmp_combo = copy.deepcopy(curr_combo)
                tmp_combo.append(feature)

                lr = LinearRegression()
                lr.fit(self.train_x[tmp_combo], self.train_y)

                score = lr.score(self.train_x[tmp_combo], self.train_y)

                # if is new max of this round, then update
                if score > best_score:
                    added_feature = feature
                    best_score = score
                    best_combo = copy.deepcopy(tmp_combo)

            curr_combo = copy.deepcopy(best_combo)
            remaining_features.remove(added_feature)  # remove added feature
            print(f"Current combination: {curr_combo}")
            print(f"Best score: {np.sqrt(best_score)}\n")

            # store in ningxiang output
            step += 1

            # only add every gap-th combo, provided it has surpassed the min features
            if step % gap == 0 and i + 1 >= min_features:

                step = 0

                self.ningxiang_output[tuple(curr_combo)] = np.sqrt(best_score)

        if len(list(self.ningxiang_output.keys())[-1]) != len(self.train_x.columns):

            self.ningxiang_output[tuple(curr_combo)] = np.sqrt(best_score)

        return self.ningxiang_output

    def get_rf_based_feature_combinations(self, min_features=0, gap=1, n_jobs=1):
        """Gets NingXiang scores based on RF feature importance"""

        if self.clf_type == None:
            raise AttributeError(
                "clf_type not found, please run set_model_type() first"
            )

        if self.train_x is None or self.train_y is None:
            raise AttributeError(
                "train_x and train_y not found, please run read_in_train_data"
            )

        # Initialise the Random Forest objects
        if self.clf_type == "Regression":
            self.rf = RandomForestRegressor(
                n_estimators=100,
                max_depth=12,
                max_features=0.75,
                random_state=self._seed,
                ccp_alpha=0,
                max_samples=0.75,
                n_jobs=n_jobs,
            )
        elif self.clf_type == "Classification":
            self.rf = RandomForestClassifier(
                n_estimators=100,
                max_depth=12,
                max_features=0.75,
                random_state=self._seed,
                ccp_alpha=0,
                max_samples=0.75,
                n_jobs=n_jobs,
            )

        print("Begin fitting Random Forest")
        # fit the model and get the feature importances
        self.rf.fit(self.train_x, self.train_y)
        print("Finished fitting Random Forest")
        self.feature_importance = {
            self.train_x.columns[i]: self.rf.feature_importances_[i]
            for i in range(len(self.train_x.columns))
        }

        # use handle (which can be used on its own) to generate the ningxiang output
        self.ningxiang_output = (
            self.get_rf_based_feature_combinations_from_feature_importance(
                self.feature_importance, min_features, gap
            )
        )

        return self.ningxiang_output

    def get_xgb_based_feature_combinations(self, min_features=0, gap=1, n_jobs=1):
        """Gets NingXiang scores based on XGB feature importance"""

        if self.clf_type == None:
            raise AttributeError(
                "clf_type not found, please run set_model_type() first"
            )

        if self.train_x is None or self.train_y is None:
            raise AttributeError(
                "train_x and train_y not found, please run read_in_train_data"
            )

        # Initialise the XGBoost objects
        if self.clf_type == "Regression":
            self.xgb = XGBRegressor(
                n_estimators=100,
                max_depth=12,
                subsample=0.75,
                random_state=self._seed,
                gamma=0,
                eta=0.01,
                colsample_bytree=0.75,
                n_jobs=n_jobs,
            )
        elif self.clf_type == "Classification":
            self.xgb = XGBClassifier(
                n_estimators=100,
                max_depth=12,
                subsample=0.75,
                random_state=self._seed,
                gamma=0,
                eta=0.01,
                colsample_bytree=0.75,
                n_jobs=n_jobs,
            )

        print("Begin fitting XGBoost")
        # fit the model and get the feature importances
        self.xgb.fit(self.train_x, self.train_y)
        print("Finished fitting XGBoost")
        self.feature_importance = {
            self.train_x.columns[i]: self.xgb.feature_importances_[i]
            for i in range(len(self.train_x.columns))
        }

        # use handle (which can be used on its own) to generate the ningxiang output
        self.ningxiang_output = (
            self.get_rf_based_feature_combinations_from_feature_importance(
                self.feature_importance, min_features, gap
            )
        )

        return self.ningxiang_output

    def get_rf_based_feature_combinations_from_feature_importance(
        self, feature_importance=None, min_features=0, gap=1
    ):
        """Takes in a dictionary of features:importance and returns feature importance"""

        feature_importance = None or self.feature_importance

        # sort features by feature importance (reversed order)
        feature_importance_sorted = self._sort_dict_reverse(feature_importance)
        # get the features in the output format that can be linked up with other JiaXing packages
        self.ningxiang_output = self._get_ningxiang_rf_output(
            feature_importance_sorted, min_features, gap
        )

        return self.ningxiang_output

    def _sort_dict_reverse(self, features):
        """Helper to sort dictionary"""

        features_list = [(key, features[key]) for key in features]
        features_list.sort(key=lambda x: x[1], reverse=True)

        features_reverse_sorted = {x[0]: x[1] for x in features_list}

        return features_reverse_sorted

    def _get_ningxiang_rf_output(self, features_reverse_sorted, min_features, gap):
        """Helper to get rf feature importance into ningxiang output format"""

        out = dict()  # NingXiang output object must be a dict

        feature_combo = list()
        score = 0

        first_time_past = True
        i = 0
        step = 0
        # Continuously add feature and its score
        for feature in features_reverse_sorted:
            feature_combo.append(feature)
            score += features_reverse_sorted[feature]

            combo = tuple(feature_combo)

            step += 1

            if step % gap == 0 and i + 1 >= min_features:  # only add every gap-th combo

                step = 0
                out[combo] = score

            i += 1

        if len(list(out.keys())[-1]) != len(features_reverse_sorted):

            out[combo] = score

        return out

    def show_rf_stats(self):
        """Shows feature importance dataframe"""

        if self.feature_importance == None:
            raise AttributeError("Please run .get_rf_based_feature_combinations()")

        print(self.feature_importance)

        if self.val_x is not None and self.val_y is not None:
            print("Validation score:", self.rf.score(self.val_x, self.val_y))

    def _set_object_saving_address(self, address):
        """Read in where to save the PuDong object"""

        self.object_saving_address = address
        print("Successfully set object output address")

    def export_ningxiang_output(self, address):
        """Export NingXiang's output object"""

        self._set_object_saving_address(address)

        # Export
        object_saving_address_split = self.object_saving_address.split(".pickle")[0]

        with open(f"{object_saving_address_split}.pickle", "wb") as f:
            pickle.dump(self.ningxiang_output, f)



def smoke_test():

    try:
        print("===Begin Smoke Test===\n")

        print("===Import Modules===")
        import JXAutoML

        print("===Import Successful===")

        print("===Import NingXiang===")

        from JXAutoML.NingXiang import NingXiang

        print("===Import NingXiang Successful===")

        print("===Import JiaoCheng===")

        from JXAutoML.JiaoCheng import JiaoCheng

        print("===Import JiaoCheng Successful===")

        print("===Import JiaoChengB===")

        from JXAutoML.JiaoChengB import JiaoChengB

        print("===Import JiaoChengB Successful===")

        print("===Import YangZhou===")

        from JXAutoML.YangZhouB import YangZhouB

        print("===Import YangZhou Successful===")

        print("===Import JiaoCheng10_CV===")

        from JXAutoML.JiaoCheng_10CV import JiaoCheng_10CV

        print("===Import JiaoCheng_10CV Successful===")

        print("===Import YangZhouB_10CV===")

        from JXAutoML.YangZhouB_10CV import YangZhouB_10CV

        print("===Import YangZhouB_10CV Successful===")

        print("======\n")
        print("===SMOKETEST PASSED===")

    except Exception as e:
        print("===SMOKETEST FAILED===")
        raise e


def test_NingXiang(X, y, mode, feature_importance_model):
    from JXAutoML.NingXiang import NingXiang

    feature_selector = NingXiang()
    feature_selector.read_in_train_data(X, y)
    feature_selector.set_model_type(mode)

    if feature_importance_model == "RF":
        feature_order_dict = feature_selector.get_rf_based_feature_combinations(
            n_jobs=-1
        )
        feature_selector.show_rf_stats()
    elif feature_importance_model == "XGB":
        feature_order_dict = feature_selector.get_xgb_based_feature_combinations(
            n_jobs=-1
        )
        feature_selector.show_rf_stats()
    else:
        feature_order_dict = feature_selector.get_lr_based_feature_combinations()

    return feature_order_dict


def test_YangZhou(
    train_X,
    train_y,
    val_X,
    val_y,
    test_X,
    test_y,
    mode,
    pytorch_model,
    feature_importance_ordering,
    TUNE_FEATURES_AS_HYPERPARAMETERS,
    optimised_metric,
    **kwargs,
):
    from JXAutoML.YangZhouB import YangZhouB as tuner

    if mode == "Regression":
        from sklearn.ensemble import RandomForestRegressor as clf
    elif mode == "Classification":
        from sklearn.ensemble import RandomForestClassifier as clf

    # what values to try for each hyperparameter
    parameter_choices = {
        "max_depth": (3, 6, 12, 24),
        "max_samples": (0.4, 0.55, 0.7, 0.85),
    }

    # what values to set non-tuneable parameters/hyperparameters
    non_tunable_hyperparameters_dict = {"random_state": 42, "n_jobs": -1}

    tuner = tuner()

    # define what model we are tuning
    tuner.read_in_model(
        clf, mode, pytorch_model=pytorch_model, optimised_metric=optimised_metric
    )

    # read in the data for training and validation
    tuner.read_in_data(train_X, train_y, val_X, val_y, test_X, test_y)

    # set what hp values to tune
    tuner.set_hyperparameters(parameter_choices)
    # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up hp values that need to be changed from default but NOT to be tuned
    tuner.set_non_tuneable_hyperparameters(non_tunable_hyperparameters_dict)

    # set up feature importance ordering

    if TUNE_FEATURES_AS_HYPERPARAMETERS:
        tuner.set_features(feature_importance_ordering)
        # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up where to save the tuning result csv
    tuner.set_tuning_result_saving_address(
        f"local_test_output/yangzhou_test_tuning_result.csv"
    )

    # set up where to save the current best model
    tuner.set_best_model_saving_address(
        f"local_test_output/yangzhou_test_tuning_best_model"
    )

    tuner.tune()


def test_JiaoCheng(
    train_X,
    train_y,
    val_X,
    val_y,
    test_X,
    test_y,
    mode,
    pytorch_model,
    feature_importance_ordering,
    TUNE_FEATURES_AS_HYPERPARAMETERS,
    optimised_metric,
    JiaoChengB,
    **kwargs,
):
    if JiaoChengB:
        from JXAutoML.JiaoChengB import JiaoChengB as tuner
    else:
        from JXAutoML.JiaoCheng import JiaoCheng as tuner
    if mode == "Regression":
        from sklearn.ensemble import RandomForestRegressor as clf
    elif mode == "Classification":
        from sklearn.ensemble import RandomForestClassifier as clf

    # what values to try for each hyperparameter
    parameter_choices = {
        "max_depth": (3, 6, 12, 24),
        "max_samples": (0.4, 0.55, 0.7, 0.85),
    }

    # what values to set non-tuneable parameters/hyperparameters
    non_tunable_hyperparameters_dict = {"random_state": 42, "n_jobs": -1}

    tuning_order = (
        ["features", "max_depth", "max_samples"]
        if TUNE_FEATURES_AS_HYPERPARAMETERS
        else ["max_depth", "max_samples"]
    )
    default_hyperparameter_values = (
        {"features": 0, "max_depth": 3, "max_samples": 0.4}
        if TUNE_FEATURES_AS_HYPERPARAMETERS
        else {"max_depth": 3, "max_samples": 0.4}
    )

    tuner = tuner()

    # define what model we are tuning
    tuner.read_in_model(
        clf, mode, pytorch_model=pytorch_model, optimised_metric=optimised_metric
    )

    # read in the data for training and validation
    tuner.read_in_data(train_X, train_y, val_X, val_y, test_X, test_y)

    # set what hp values to tune
    tuner.set_hyperparameters(parameter_choices)
    # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up hp values that need to be changed from default but NOT to be tuned
    tuner.set_non_tuneable_hyperparameters(non_tunable_hyperparameters_dict)

    # set up feature importance ordering

    if TUNE_FEATURES_AS_HYPERPARAMETERS:
        tuner.set_features(feature_importance_ordering)
        # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up the order of hyperparameters when iteratively tuning using JiaoCheng
    tuner.set_tuning_order(tuning_order)

    # set up the default hp values for first iteration of tuning JiaoCheng
    tuner.set_hyperparameter_default_values(default_hyperparameter_values)

    # set up where to save the tuning result csv
    tuner.set_tuning_result_saving_address(
        f'local_test_output/jiaocheng{"b" if JiaoChengB else ""}_test_tuning_result.csv'
    )

    # set up where to save the current best model
    tuner.set_best_model_saving_address(
        f'local_test_output/jiaocheng{"b" if JiaoChengB else ""}_test_tuning_best_model'
    )

    tuner.tune()


def test_YangZhou_10CV(
    train_X,
    train_y,
    val_X,
    val_y,
    test_X,
    test_y,
    mode,
    pytorch_model,
    feature_importance_ordering,
    TUNE_FEATURES_AS_HYPERPARAMETERS,
    optimised_metric,
    **kwargs,
):
    from JXAutoML.YangZhouB_10CV import YangZhouB_10CV as tuner

    if mode == "Regression":
        from sklearn.ensemble import RandomForestRegressor as clf
    elif mode == "Classification":
        from sklearn.ensemble import RandomForestClassifier as clf

    from sklearn.model_selection import KFold

    train_X = train_X.reset_index(drop=True)
    train_y = train_y.reset_index(drop=True)
    kf = KFold(n_splits=10, shuffle=True, random_state=42)
    cv_splits = list(kf.split(train_X, train_y))

    train_X_list = [train_X.iloc[train_idx] for train_idx, _ in cv_splits]
    train_y_list = [train_y.iloc[train_idx] for train_idx, _ in cv_splits]

    val_X_list = [train_X.iloc[val_idx] for _, val_idx in cv_splits]
    val_y_list = [train_y.iloc[val_idx] for _, val_idx in cv_splits]

    # what values to try for each hyperparameter
    parameter_choices = {
        "max_depth": (3, 6, 12, 24),
        "max_samples": (0.4, 0.55, 0.7, 0.85),
    }

    # what values to set non-tuneable parameters/hyperparameters
    non_tunable_hyperparameters_dict = {"random_state": 42, "n_jobs": -1}

    tuner = tuner()

    # define what model we are tuning
    tuner.read_in_model(
        clf, mode, pytorch_model=pytorch_model, optimised_metric=optimised_metric
    )

    # read in the data for training and validation
    tuner.read_in_data(train_X_list, train_y_list, val_X_list, val_y_list)

    # set what hp values to tune
    tuner.set_hyperparameters(parameter_choices)
    # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up hp values that need to be changed from default but NOT to be tuned
    tuner.set_non_tuneable_hyperparameters(non_tunable_hyperparameters_dict)

    # set up feature importance ordering

    if TUNE_FEATURES_AS_HYPERPARAMETERS:
        tuner.set_features(feature_importance_ordering)
        # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up where to save the tuning result csv
    tuner.set_tuning_result_saving_address(
        f"local_test_output/yangzhou_test_tuning_result.csv"
    )

    # set up where to save the current best model
    tuner.set_best_model_saving_address(
        f"local_test_output/yangzhou_test_tuning_best_model"
    )

    tuner.tune()


def test_JiaoCheng_10CV(
    train_X,
    train_y,
    val_X,
    val_y,
    test_X,
    test_y,
    mode,
    pytorch_model,
    feature_importance_ordering,
    TUNE_FEATURES_AS_HYPERPARAMETERS,
    optimised_metric,
    **kwargs,
):

    from JXAutoML.JiaoCheng_10CV import JiaoCheng_10CV as tuner

    if mode == "Regression":
        from sklearn.ensemble import RandomForestRegressor as clf
    elif mode == "Classification":
        from sklearn.ensemble import RandomForestClassifier as clf

    from sklearn.model_selection import KFold

    train_X = train_X.reset_index(drop=True)
    train_y = train_y.reset_index(drop=True)
    kf = KFold(n_splits=10, shuffle=True, random_state=42)
    cv_splits = list(kf.split(train_X, train_y))

    train_X_list = [train_X.iloc[train_idx] for train_idx, _ in cv_splits]
    train_y_list = [train_y.iloc[train_idx] for train_idx, _ in cv_splits]

    val_X_list = [train_X.iloc[val_idx] for _, val_idx in cv_splits]
    val_y_list = [train_y.iloc[val_idx] for _, val_idx in cv_splits]

    # what values to try for each hyperparameter
    parameter_choices = {
        "max_depth": (3, 6, 12, 24),
        "max_samples": (0.4, 0.55, 0.7, 0.85),
    }

    # what values to set non-tuneable parameters/hyperparameters
    non_tunable_hyperparameters_dict = {"random_state": 42, "n_jobs": -1}

    tuning_order = (
        ["features", "max_depth", "max_samples"]
        if TUNE_FEATURES_AS_HYPERPARAMETERS
        else ["max_depth", "max_samples"]
    )
    default_hyperparameter_values = (
        {"features": 0, "max_depth": 3, "max_samples": 0.4}
        if TUNE_FEATURES_AS_HYPERPARAMETERS
        else {"max_depth": 3, "max_samples": 0.4}
    )

    tuner = tuner()

    # define what model we are tuning
    tuner.read_in_model(
        clf, mode, pytorch_model=pytorch_model, optimised_metric=optimised_metric
    )

    # read in the data for training and validation
    tuner.read_in_data(train_X_list, train_y_list, val_X_list, val_y_list)

    # set what hp values to tune
    tuner.set_hyperparameters(parameter_choices)
    # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up hp values that need to be changed from default but NOT to be tuned
    tuner.set_non_tuneable_hyperparameters(non_tunable_hyperparameters_dict)

    # set up feature importance ordering

    if TUNE_FEATURES_AS_HYPERPARAMETERS:
        tuner.set_features(feature_importance_ordering)
        # WARNING: this may take a while if no. tuneable hyperparameters are large

    # set up the order of hyperparameters when iteratively tuning using JiaoCheng
    tuner.set_tuning_order(tuning_order)

    # set up the default hp values for first iteration of tuning JiaoCheng
    tuner.set_hyperparameter_default_values(default_hyperparameter_values)

    # set up where to save the tuning result csv
    tuner.set_tuning_result_saving_address(
        f"local_test_output/jiaocheng_10cv_test_tuning_result.csv"
    )

    # set up where to save the current best model
    tuner.set_best_model_saving_address(
        f"local_test_output/jiaocheng10cv_test_tuning_best_model"
    )

    tuner.tune()


def local_test():
    # Create a regression dataset
    from sklearn.datasets import make_regression, make_classification
    from sklearn.model_selection import train_test_split
    import pandas as pd

    X_reg, y_reg = make_regression(
        n_samples=100, n_features=5, noise=0.1, random_state=42
    )
    X_reg_df = pd.DataFrame(
        X_reg, columns=[f"feature_{i+1}" for i in range(X_reg.shape[1])]
    )
    y_reg_series = pd.Series(y_reg, name="target")

    X_reg_df_train, X_reg_df_valtest, y_reg_series_train, y_reg_series_valtest = (
        train_test_split(X_reg_df, y_reg_series, test_size=0.3, random_state=42)
    )
    X_reg_df_val, X_reg_df_test, y_reg_series_val, y_reg_series_test = train_test_split(
        X_reg_df_valtest, y_reg_series_valtest, test_size=0.5, random_state=42
    )

    # Create a classification dataset with 2 classes
    X_class_2, y_class_2 = make_classification(
        n_samples=100,
        n_features=5,
        n_classes=2,
        n_clusters_per_class=1,
        random_state=42,
    )
    X_class_2_df = pd.DataFrame(
        X_class_2, columns=[f"feature_{i+1}" for i in range(X_class_2.shape[1])]
    )
    y_class_2_series = pd.Series(y_class_2, name="target")

    (
        X_class_2_df_train,
        X_class_2_df_valtest,
        y_class_2_series_train,
        y_class_2_series_valtest,
    ) = train_test_split(X_class_2_df, y_class_2_series, test_size=0.3, random_state=42)
    X_class_2_df_val, X_class_2_df_test, y_class_2_series_val, y_class_2_series_test = (
        train_test_split(
            X_class_2_df_valtest,
            y_class_2_series_valtest,
            test_size=0.5,
            random_state=42,
        )
    )

    smoke_test()

    print("===Begin NingXiang Test===\n")
    print("===Begin NingXiang XGB Regression Test===")
    reg_feature_importance = test_NingXiang(
        X_class_2_df_train, y_class_2_series_train, "Regression", "XGB"
    )
    print("===Test NingXiang XGB Regression Successful===")
    print("===Begin NingXiang XGB Classification Test===")
    class_2_feature_importance = test_NingXiang(
        X_class_2_df_val, y_class_2_series_val, "Classification", "XGB"
    )
    print("===Test NingXiang XGB Classification Successful===\n")
    print("===Begin NingXiang LR Regression Test===")
    reg_feature_importance = test_NingXiang(
        X_class_2_df_train, y_class_2_series_train, "Regression", "LR"
    )
    print("===Test NingXiang LR Regression Successful===")
    print("===Begin NingXiang RF Regression Test===")
    reg_feature_importance = test_NingXiang(
        X_class_2_df_train, y_class_2_series_train, "Regression", "RF"
    )
    print("===Test NingXiang RF Regression Successful===")
    print("===Begin NingXiang RF Classification Test===")
    class_2_feature_importance = test_NingXiang(
        X_class_2_df_val, y_class_2_series_val, "Classification", "RF"
    )
    print("===Test NingXiang RF Classification Successful===\n")
    print("===Test NingXiang Successful===\n\n")

    print("===Begin YangZhou Test===\n")
    print("===Begin YangZhou Regression (No Features) Test===")
    YangZhou_test_json_reg = {
        "train_X": X_reg_df_train,
        "train_y": y_reg_series_train,
        "val_X": X_reg_df_val,
        "val_y": y_reg_series_val,
        "test_X": X_reg_df_test,
        "test_y": y_reg_series_test,
        "mode": "Regression",
        "pytorch_model": False,
        "feature_importance_ordering": reg_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": False,
        "optimised_metric": "r2",
    }
    test_YangZhou(**YangZhou_test_json_reg)
    print("===Test YangZhou Regression (No Features) Successful===\n")

    print("===Begin YangZhou Classification (With Features) Test===")
    YangZhou_test_json_class_2 = {
        "train_X": X_class_2_df_train,
        "train_y": y_class_2_series_train,
        "val_X": X_class_2_df_val,
        "val_y": y_class_2_series_val,
        "test_X": X_class_2_df_test,
        "test_y": y_class_2_series_test,
        "mode": "Classification",
        "pytorch_model": False,
        "feature_importance_ordering": class_2_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": True,
        "optimised_metric": "accuracy",
    }
    test_YangZhou(**YangZhou_test_json_class_2)
    print("===Test YangZhou Classification (With Features) Successful===\n")

    print("===Test YangZhou Successful===\n\n")

    print("===Begin JiaoCheng Test===\n")

    print("===Begin JiaoCheng Regression (No Features) Test===")
    JiaoCheng_test_json_reg = {
        "train_X": X_reg_df_train,
        "train_y": y_reg_series_train,
        "val_X": X_reg_df_val,
        "val_y": y_reg_series_val,
        "test_X": X_reg_df_test,
        "test_y": y_reg_series_test,
        "mode": "Regression",
        "pytorch_model": False,
        "feature_importance_ordering": reg_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": False,
        "optimised_metric": "r2",
        "JiaoChengB": False,
    }
    test_JiaoCheng(**JiaoCheng_test_json_reg)
    print("===Test JiaoCheng Regression (No Features) Successful===")

    print("===Begin JiaoCheng Classification (With Features) Test===")
    JiaoCheng_test_json_class_2 = {
        "train_X": X_class_2_df_train,
        "train_y": y_class_2_series_train,
        "val_X": X_class_2_df_val,
        "val_y": y_class_2_series_val,
        "test_X": X_class_2_df_test,
        "test_y": y_class_2_series_test,
        "mode": "Classification",
        "pytorch_model": False,
        "feature_importance_ordering": class_2_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": True,
        "optimised_metric": "accuracy",
        "JiaoChengB": False,
    }
    test_JiaoCheng(**JiaoCheng_test_json_class_2)
    print("===Test JiaoCheng Classification (With Features) Successful===\n")

    print("===Test JiaoCheng Successful===\n\n")

    print("===Begin JiaoChengB Test===\n")

    print("===Begin JiaoChengB Regression (With Features) Test===")
    JiaoChengB_test_json_reg = {
        "train_X": X_reg_df_train,
        "train_y": y_reg_series_train,
        "val_X": X_reg_df_val,
        "val_y": y_reg_series_val,
        "test_X": X_reg_df_test,
        "test_y": y_reg_series_test,
        "mode": "Regression",
        "pytorch_model": False,
        "feature_importance_ordering": reg_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": True,
        "optimised_metric": "r2",
        "JiaoChengB": True,
    }
    test_JiaoCheng(**JiaoChengB_test_json_reg)
    print("===Test JiaoChengB Regression (With Features) Successful===")

    print("===Begin JiaoChengB Classification (No Features) Test===")
    JiaoChengB_test_json_class_2 = {
        "train_X": X_class_2_df_train,
        "train_y": y_class_2_series_train,
        "val_X": X_class_2_df_val,
        "val_y": y_class_2_series_val,
        "test_X": X_class_2_df_test,
        "test_y": y_class_2_series_test,
        "mode": "Classification",
        "pytorch_model": False,
        "feature_importance_ordering": class_2_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": False,
        "optimised_metric": "accuracy",
        "JiaoChengB": True,
    }
    test_JiaoCheng(**JiaoChengB_test_json_class_2)
    print("===Test JiaoChengB Classification (No Features) Successful===\n")

    print("===Test JiaoChengB Successful===\n\n")

    print("===Begin YangZhou_10CV Test===\n")
    print("===Begin YangZhou_10CV Regression (No Features) Test===")
    YangZhou10CV_test_json_reg = {
        "train_X": X_reg_df_train,
        "train_y": y_reg_series_train,
        "val_X": X_reg_df_val,
        "val_y": y_reg_series_val,
        "test_X": X_reg_df_test,
        "test_y": y_reg_series_test,
        "mode": "Regression",
        "pytorch_model": False,
        "feature_importance_ordering": reg_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": False,
        "optimised_metric": "r2",
    }
    test_YangZhou_10CV(**YangZhou10CV_test_json_reg)
    print("===Test YangZhou_10CV Regression (No Features) Successful===\n")

    print("===Begin YangZhou_10CV Classification (With Features) Test===")
    YangZhou10CV_test_json_class_2 = {
        "train_X": X_class_2_df_train,
        "train_y": y_class_2_series_train,
        "val_X": X_class_2_df_val,
        "val_y": y_class_2_series_val,
        "test_X": X_class_2_df_test,
        "test_y": y_class_2_series_test,
        "mode": "Classification",
        "pytorch_model": False,
        "feature_importance_ordering": class_2_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": True,
        "optimised_metric": "accuracy",
    }
    test_YangZhou_10CV(**YangZhou10CV_test_json_class_2)
    print("===Test YangZhou_10CV Classification (With Features) Successful===\n")

    print("===Test YangZhou_10CV Successful===\n\n")

    print("===Begin JiaoCheng_10CV Test===\n")

    print("===Begin JiaoCheng_10CV Regression (No Features) Test===")
    JiaoCheng10CV_test_json_reg = {
        "train_X": X_reg_df_train,
        "train_y": y_reg_series_train,
        "val_X": X_reg_df_val,
        "val_y": y_reg_series_val,
        "test_X": X_reg_df_test,
        "test_y": y_reg_series_test,
        "mode": "Regression",
        "pytorch_model": False,
        "feature_importance_ordering": reg_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": False,
        "optimised_metric": "r2",
        "JiaoChengB": False,
    }
    test_JiaoCheng_10CV(**JiaoCheng10CV_test_json_reg)
    print("===Test JiaoCheng_10CV Regression (No Features) Successful===")

    print("===Begin JiaoCheng_10CV Classification (With Features) Test===")
    JiaoCheng10CV_test_json_class_2 = {
        "train_X": X_class_2_df_train,
        "train_y": y_class_2_series_train,
        "val_X": X_class_2_df_val,
        "val_y": y_class_2_series_val,
        "test_X": X_class_2_df_test,
        "test_y": y_class_2_series_test,
        "mode": "Classification",
        "pytorch_model": False,
        "feature_importance_ordering": class_2_feature_importance,
        "TUNE_FEATURES_AS_HYPERPARAMETERS": True,
        "optimised_metric": "accuracy",
        "JiaoChengB": False,
    }
    test_JiaoCheng_10CV(**JiaoCheng10CV_test_json_class_2)
    print("===Test JiaoCheng_10CV Classification (With Features) Successful===\n")

    print("===Test JiaoCheng_10CV Successful===\n\n")

    print("===")


if __name__ == "__main__":
    local_test()
    print("ALL TESTS PASSED!")



from setuptools import setup, find_packages

setup(
    name="JXAutoML",  # Replace with your package name
    version="0.4.4",
    author="Lang Chen",
    author_email="ronchen6666@gmail.com",
    description="Advanced HyperParameter Tuning",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    # Replace with your GitHub repo
    url="https://github.com/TGChenZP/JXAutoML",
    packages=find_packages(),  # Automatically find all packages
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
